@@ -5,13 +5,13 @@ from typing import (
 )
 from collections.abc import Mapping
 from collections import OrderedDict, defaultdict
-import operator
-from functools import reduce
 import json
+import copy
 from enum import Enum
 from multimethod import multidispatch, multimethod
 from .yaml import yaml_initializer, to_yaml, from_yaml
 from .strings import is_match_string
+import snoop
 
 class DictItem(str, Enum):
   KEY = "key"
@@ -46,10 +46,19 @@ class DictFactory(dict):
     from_yaml = from_yaml
 
     """ Factory class for custom dictionary """
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self,
+            *args: Any,
+            as_default_dict: bool=False,
+            **kwargs: Any
+        ):
+        if as_default_dict:
+            new = self.from_dict(dict(*args, **kwargs), factory=type(self))
+            super().__init__(new)
+        else:
+            super().__init__(*args, **kwargs)
         self.yaml_initializer()
 
+
     def __repr__(self):
         return '{0}({1})'.format(self.__class__.__name__, dict.__repr__(self))
 
@@ -67,6 +76,9 @@ class DictFactory(dict):
         for key, val in dict(*args, **kwargs).items():
             self[key] = val
 
+    def copy(self):
+        return copy.copy(self)
+
     def get(self, key: Hashable, default=None):
         if key not in self:
             return default
@@ -211,8 +223,8 @@ class DictFactory(dict):
                 return obj
 
         def post_convert(partial, obj):
-            if isinstance(obj, iDict):
-                partial = iDict((key, convert_loop(obj[key]))
+            if isinstance(obj, Mapping) and factory == type(self):
+                partial = factory((key, convert_loop(obj[key]))
                                 for key in obj.keys() )
             elif isinstance(obj, Mapping):
                 partial.update((key, convert_loop(obj[key]))
@@ -245,17 +257,6 @@ class DictFactory(dict):
 
 class aDict(DictFactory):
 
-    def __init__(self,
-            *args: Any,
-            as_default_dict: bool=False,
-            **kwargs: Any
-        ):
-        if as_default_dict:
-            self.update(*args, **kwargs)
-        else:
-            super().__init__(*args, **kwargs)
-        self.yaml_initializer()
-
     def __str__(self):
         return '{}'.format(self.to_dict(self))
 
@@ -293,10 +294,6 @@ class aDict(DictFactory):
         else:
             object.__delattr__(self, k)
 
-    @property
-    def __dict__(self):
-        return self.to_dict(self)
-
     def update(self, *args, **kwargs):
         for key, val in dict(*args, **kwargs).items():
             if isinstance(val, Mapping) and not isinstance(val, type(self)):
@@ -304,8 +301,6 @@ class aDict(DictFactory):
             else:
                 self[key] = val
 
-    def copy(self):
-        return self.from_dict(self)
 
 
 class uDict(DictFactory):
@@ -877,36 +872,25 @@ class uDict(DictFactory):
 
 class iDict(DictFactory):
 
+    def _blocked_attribute(self):
+        raise AttributeError( ( "A {} cannot be modified."
+                                .format(self.__class__.__name__) ) )
+
+    _blocked_attribute = property(_blocked_attribute)
+
+    __delitem__ = __setitem__ = _blocked_attribute
+    pop = popitem = setdefault = clear = update = _blocked_attribute
+
+
     def __missing__(self, key):
         return None
 
-    def __getattr__(self, attribute):
-        if attribute in ('clear', 'update', 'pop', 'popitem', 'setdefault'):
-            raise AttributeError(
-                r"{} object has no attribute {}"
-                .format(type(self).__name__, attribute) )
-
-    def __setitem__(self, key, value):
-        raise TypeError(
-            r"{} object does not support item assignment"
-            .format(type(self).__name__) )
-
-    def __delitem__(self, key):
-        raise TypeError(
-            r"{} object does not support item deletion"
-            .format(type(self).__name__) )
-
     def __getattribute__(self, attribute):
-        if attribute in ('clear', 'update', 'pop', 'popitem', 'setdefault'):
-            raise AttributeError(
-                r"{} object has no attribute {}"
-                .format(type(self).__name__, attribute) )
         return dict.__getattribute__(self, attribute)
 
     def __hash__(self):
         return hash(tuple(sorted(self.items())))
 
-
     def fromkeys(self,
             seq: Sequence,
             value: Any,
